@@ -92,6 +92,7 @@ ephy_dnd_drag_data_get (GtkWidget *widget,
 	{
 		result = g_string_new (NULL);
                 (* each_selected_item_iterator) (add_one_node, container_context, result);
+		g_string_erase (result, result->len - 1, -1);
 	}
 	else
 	{
@@ -116,15 +117,19 @@ ephy_dnd_node_list_extract_nodes (const char *node_list)
 
 	nodes = g_strsplit (node_list, ";", -1);
 
-	for (i = 0; nodes[i] != NULL; i++)
+	for (i = 0; nodes[i] != NULL; i = i + 2)
 	{
 		gulong id;
+		EphyNodeDb *db;
 
-		if (ephy_str_to_int (nodes[i], &id))
+		db = ephy_node_db_get_by_name (nodes[i]);
+		g_return_val_if_fail (db != NULL, NULL);
+
+		if (ephy_string_to_int (nodes[i + 1], &id))
 		{
 			EphyNode *node;
 
-			node = ephy_node_get_from_id (id);
+			node = ephy_node_db_get_node_from_id (db, id);
 			g_return_val_if_fail (node != NULL, NULL);
 			result = g_list_append (result, node);
 		}
