@@ -20,8 +20,10 @@
 #define EPHY_TAB_H
 
 #include "ephy-embed.h"
+#include "egg-action.h"
 
 #include <glib-object.h>
+#include <gtk/gtkwidget.h>
 
 G_BEGIN_DECLS
 
@@ -32,21 +34,24 @@ typedef struct EphyTabClass EphyTabClass;
 #define EPHY_TAB_CLASS(klass)     (GTK_CHECK_CLASS_CAST ((klass), EPHY_TAB, EphyTabClass))
 #define IS_EPHY_TAB(obj)          (GTK_CHECK_TYPE ((obj), EPHY_TAB_TYPE))
 #define IS_EPHY_TAB_CLASS(klass)  (GTK_CHECK_CLASS_TYPE ((klass), EPHY_TAB))
+#define EPHY_TAB_GET_CLASS(obj)	  (G_TYPE_INSTANCE_GET_CLASS((obj), EPHY_TAB_TYPE, EphyTabClass))
 
 typedef struct EphyTab EphyTab;
 typedef struct EphyTabPrivate EphyTabPrivate;
 
 typedef enum
 {
-	TAB_CONTROL_TITLE
-} TabControlID;
+	TAB_NAV_UP	= 1 << 0,
+	TAB_NAV_BACK	= 1 << 1,
+	TAB_NAV_FORWARD	= 1 << 2,
+} TabNavigationFlags;
 
 typedef enum
 {
-	TAB_LOAD_NONE,
-	TAB_LOAD_STARTED,
-	TAB_LOAD_COMPLETED
-} TabLoadStatus;
+	TAB_ADDRESS_EXPIRE_NOW,
+	TAB_ADDRESS_EXPIRE_NEXT,
+	TAB_ADDRESS_EXPIRE_CURRENT
+} TabAddressExpire;
 
 struct EphyTab
 {
@@ -56,53 +61,58 @@ struct EphyTab
 
 struct EphyTabClass
 {
-        GObjectClass parent_class;
+	GObjectClass parent_class;
 };
 
 /* Include the header down here to resolve circular dependency */
 #include "ephy-window.h"
 
-GType         ephy_tab_get_type			(void);
+GType			ephy_tab_get_type		(void);
 
-EphyTab      *ephy_tab_new			(void);
+EphyTab	*		ephy_tab_new			(void);
 
-EphyEmbed    *ephy_tab_get_embed		(EphyTab *tab);
+EggAction *		ephy_tab_get_action		(EphyTab *tab);
 
-void          ephy_tab_set_window		(EphyTab *tab,
-						 EphyWindow *window);
+EphyEmbed *		ephy_tab_get_embed		(EphyTab *tab);
 
-EphyWindow   *ephy_tab_get_window		(EphyTab *tab);
+void			ephy_tab_set_window		(EphyTab *tab,
+							 EphyWindow *window);
 
-void	      ephy_tab_set_is_active		(EphyTab *tab,
-						 gboolean is_active);
+EphyWindow *		ephy_tab_get_window		(EphyTab *tab);
 
-gboolean      ephy_tab_get_is_active		(EphyTab *tab);
+const char *		ephy_tab_get_icon_address	(EphyTab *tab);
 
-gboolean      ephy_tab_get_visibility           (EphyTab *tab);
+gboolean		ephy_tab_get_load_status	(EphyTab *tab);
 
-TabLoadStatus ephy_tab_get_load_status		(EphyTab *tab);
+const char *		ephy_tab_get_link_message	(EphyTab *tab);
 
-int	      ephy_tab_get_load_percent		(EphyTab *tab);
 
-const char   *ephy_tab_get_status_message	(EphyTab *tab);
+int			ephy_tab_get_load_percent	(EphyTab *tab);
 
-const char   *ephy_tab_get_title		(EphyTab *tab);
+void			ephy_tab_set_location		(EphyTab *tab,
+							 const char *location,
+							 TabAddressExpire expire);
 
-const char   *ephy_tab_get_location             (EphyTab *tab);
+const char *		ephy_tab_get_location		(EphyTab *tab);
 
-const char   *ephy_tab_get_favicon_url          (EphyTab *tab);
+TabNavigationFlags	ephy_tab_get_navigation_flags	(EphyTab *tab);
 
-void	      ephy_tab_set_location             (EphyTab *tab,
-						 char *location);
+EmbedSecurityLevel	ephy_tab_get_security_level	(EphyTab *tab);
 
-void	      ephy_tab_get_size			(EphyTab *tab,
-						 int *width,
-						 int *height);
+void			ephy_tab_get_size		(EphyTab *tab,
+							 int *width,
+							 int *height);
 
-void	      ephy_tab_update_control		(EphyTab *tab,
-						 TabControlID id);
+const char *		ephy_tab_get_status_message	(EphyTab *tab);
 
-EphyEmbedEvent *ephy_tab_get_event		(EphyTab *tab);
+const char *		ephy_tab_get_title		(EphyTab *tab);
+
+void			ephy_tab_set_visibility		(EphyTab *tab,
+							 gboolean visible);
+
+gboolean		ephy_tab_get_visibility		(EphyTab *tab);
+
+float			ephy_tab_get_zoom		(EphyTab *tab);
 
 G_END_DECLS
 
