@@ -17,6 +17,10 @@
  *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  */
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
 #include "toolbar.h"
 #include "egg-menu-merge.h"
 #include "ephy-file-helpers.h"
@@ -31,11 +35,13 @@
 #include "ephy-go-action.h"
 #include "ephy-navigation-action.h"
 #include "ephy-bookmark-action.h"
+#include "ephy-zoom-action.h"
 #include "window-commands.h"
 #include "ephy-string.h"
 #include "ephy-debug.h"
 #include "ephy-new-bookmark.h"
 #include "ephy-stock-icons.h"
+#include "eggtoolbar.h"
 
 #include <string.h>
 
@@ -75,7 +81,7 @@ struct ToolbarPrivate
 	EggMenuMerge *ui_merge;
 	EggActionGroup *action_group;
 	gboolean visibility;
-	GtkWidget *location_entry;
+	gboolean can_set_location;
 	GtkWidget *spinner;
 	GtkWidget *favicon;
 	GtkWidget *go;
@@ -113,30 +119,59 @@ toolbar_get_type (void)
 static void
 go_location_cb (EggAction *action, char *location, EphyWindow *window)
 {
-	EphyEmbed *embed;
-
-	embed = ephy_window_get_active_embed (window);
-	g_return_if_fail (embed != NULL);
-
-	ephy_embed_load_url (embed, location);
+	ephy_window_load_url (window, location);
 }
 
+static void
+zoom_to_level_cb (EggAction *action, float zoom, EphyWindow *window)
+{
+	ephy_window_set_zoom (window, zoom);
+}
 
-static EggAction *
-ensure_bookmark_action (Toolbar *t, EphyBookmarks *bookmarks, gulong id, const char *action_name)
+static void
+topic_destroy_cb (EphyNode *node,
+		  Toolbar *t)
 {
 	EggAction *action;
+	char *name;
+	EphyToolbarsModel *model;
+	long id;
 
-	LOG ("Creating action for bookmark id %ld", id)
+	model = ephy_shell_get_toolbars_model (ephy_shell);
 
-	action = ephy_bookmark_action_new (action_name, id);
+	id = ephy_node_get_id (node);
+	name = g_strdup_printf ("GoTopicId%ld", ephy_node_get_id (node));
+	action = egg_action_group_get_action (t->priv->action_group, name);
+	if (action)
+	{
+		ephy_toolbars_model_remove_bookmark (model, TRUE, id);
+		egg_action_group_remove_action (t->priv->action_group, action);
+	}
 
-	g_signal_connect (action, "go_location",
-			  G_CALLBACK (go_location_cb), t->priv->window);
-	egg_action_group_add_action (t->priv->action_group, action);
-	g_object_unref (action);
+	g_free (name);
+}
 
-	return action;
+static void
+bookmark_destroy_cb (EphyNode *node,
+		     Toolbar *t)
+{
+	EggAction *action;
+	char *name;
+	EphyToolbarsModel *model;
+	long id;
+
+	model = ephy_shell_get_toolbars_model (ephy_shell);
+
+	id = ephy_node_get_id (node);
+	name = g_strdup_printf ("GoBookmarkId%ld", id);
+	action = egg_action_group_get_action (t->priv->action_group, name);
+	if (action)
+	{
+		ephy_toolbars_model_remove_bookmark (model, FALSE, id);
+		egg_action_group_remove_action (t->priv->action_group, action);
+	}
+
+	g_free (name);
 }
 
 static void
@@ -144,7 +179,7 @@ toolbar_ensure_action (Toolbar *t,
 		       const char *name)
 {
 	EggAction *action = NULL;
-	gulong id = 0;
+	long id = 0;
 	EphyBookmarks *bookmarks;
 
 	LOG ("Ensure action %s", name)
@@ -153,18 +188,32 @@ toolbar_ensure_action (Toolbar *t,
 
 	if (g_str_has_prefix (name, "GoBookmarkId"))
 	{
-		if (!ephy_str_to_int (name + strlen ("GoBookmarkId"), &id))
+		EphyNode *node;
+
+		if (!ephy_string_to_int (name + strlen ("GoBookmarkId"), &id))
 		{
 			return;
 		}
 
 		LOG ("Create action %s", name)
 
-		action = ensure_bookmark_action (t, bookmarks, id, name);
+		action = ephy_bookmark_action_new (name, id);
+		g_signal_connect (action, "go_location",
+				  G_CALLBACK (go_location_cb), t->priv->window);
+		egg_action_group_add_action (t->priv->action_group, action);
+		g_object_unref (action);
+
+		node = ephy_bookmarks_get_from_id (bookmarks, id);
+		ephy_node_signal_connect_object (node,
+					         EPHY_NODE_DESTROY,
+					         (EphyNodeCallback) bookmark_destroy_cb,
+					         G_OBJECT (t));
 	}
 	else if (g_str_has_prefix (name, "GoTopicId"))
 	{
-		if (!ephy_str_to_int (name + strlen ("GoTopicId"), &id))
+		EphyNode *node;
+
+		if (!ephy_string_to_int (name + strlen ("GoTopicId"), &id))
 		{
 			return;
 		}
@@ -177,6 +226,12 @@ toolbar_ensure_action (Toolbar *t,
 				  t->priv->window);
 		egg_action_group_add_action (t->priv->action_group, action);
 		g_object_unref (action);
+
+		node = ephy_bookmarks_get_from_id (bookmarks, id);
+		ephy_node_signal_connect_object (node,
+					         EPHY_NODE_DESTROY,
+					         (EphyNodeCallback) topic_destroy_cb,
+					         G_OBJECT (t));
 	}
 }
 
@@ -279,14 +334,17 @@ toolbar_setup_actions (Toolbar *t)
 
 	action = g_object_new (EPHY_TYPE_SPINNER_ACTION,
 			       "name", "Spinner",
-			       "label", "Spinner",
+			       "label", _("Spinner"),
 			       NULL);
 	egg_action_group_add_action (t->priv->action_group, action);
 	g_object_unref (action);
 
+	/* FIXME: I'm still waiting for the exact term to 
+	 * user here from the docs team.
+	 */
 	action = g_object_new (EPHY_TYPE_LOCATION_ACTION,
 			       "name", "Location",
-			       "label", "Location",
+			       "label", _("Address Entry"),
 			       "stock_id", EPHY_STOCK_ENTRY,
 			       NULL);
 	g_signal_connect (action, "go_location",
@@ -294,9 +352,19 @@ toolbar_setup_actions (Toolbar *t)
 	egg_action_group_add_action (t->priv->action_group, action);
 	g_object_unref (action);
 
+	action = g_object_new (EPHY_TYPE_ZOOM_ACTION,
+			       "name", "Zoom",
+			       "label", _("Zoom"),
+			       "zoom", 1.0,
+			       NULL);
+	g_signal_connect (action, "zoom_to_level",
+			  G_CALLBACK (zoom_to_level_cb), t->priv->window);
+	egg_action_group_add_action (t->priv->action_group, action);
+	g_object_unref (action);
+
 	action = g_object_new (EPHY_TYPE_FAVICON_ACTION,
 			       "name", "Favicon",
-			       "label", "Favicon",
+			       "label", _("Favicon"),
 			       "window", t->priv->window,
 			       NULL);
 	egg_action_group_add_action (t->priv->action_group, action);
@@ -304,7 +372,7 @@ toolbar_setup_actions (Toolbar *t)
 
 	action = g_object_new (EPHY_TYPE_GO_ACTION,
 			       "name", "ToolbarGo",
-			       "label", "Go",
+			       "label", _("Go"),
 			       "stock_id", GTK_STOCK_JUMP_TO,
 			       NULL);
 	g_signal_connect (action, "activate",
@@ -352,6 +420,68 @@ init_bookmarks_toolbar (Toolbar *t)
 	}
 }
 
+static void
+update_toolbar_remove_flag (EphyToolbarsModel *model, gpointer data)
+{
+	int i, n_toolbars;
+	int not_removable = 0;
+
+	n_toolbars = egg_toolbars_model_n_toolbars
+		(EGG_TOOLBARS_MODEL (model));
+
+	/* If there is only one toolbar and the bookmarks bar */
+	if (n_toolbars <= 2)
+	{
+		not_removable = EGG_TB_MODEL_NOT_REMOVABLE;
+	}
+
+	for (i = 0; i < n_toolbars; i++)
+	{
+		const char *t_name;
+
+		t_name = egg_toolbars_model_toolbar_nth
+			(EGG_TOOLBARS_MODEL (model), i);
+		g_return_if_fail (t_name != NULL);
+
+		if (!(strcmp (t_name, "BookmarksBar") == 0))
+		{
+			egg_toolbars_model_set_flags
+				(EGG_TOOLBARS_MODEL (model),
+				 not_removable, i);
+		}
+	}
+}
+
+static GtkWidget *
+get_location_entry (Toolbar *t)
+{
+	EggAction *action;
+	GtkWidget *location;
+
+	action = egg_action_group_get_action
+		(t->priv->action_group, "Location");
+	location = ephy_location_action_get_widget
+		(EPHY_LOCATION_ACTION (action));
+
+	return location;
+}
+
+static void
+location_user_changed_cb (GtkWidget *entry, Toolbar *t)
+{
+	EphyTab *tab;
+	char *address;
+
+	tab = ephy_window_get_active_tab (t->priv->window);
+	g_return_if_fail (IS_EPHY_TAB (tab));
+
+	t->priv->can_set_location = FALSE;
+	address = ephy_location_entry_get_location (EPHY_LOCATION_ENTRY (entry));
+	ephy_tab_set_location (tab, address, TAB_ADDRESS_EXPIRE_CURRENT);
+	g_free (address);
+	t->priv->can_set_location = TRUE;
+}
+
 static void
 toolbar_set_window (Toolbar *t, EphyWindow *window)
 {
@@ -371,101 +501,32 @@ toolbar_set_window (Toolbar *t, EphyWindow *window)
 			  NULL);
 
 	model = ephy_shell_get_toolbars_model (ephy_shell);
+	g_signal_connect (EGG_TOOLBARS_MODEL (model), "toolbar_added",
+			  G_CALLBACK (update_toolbar_remove_flag),
+			  NULL);
+	g_signal_connect (EGG_TOOLBARS_MODEL (model), "toolbar_removed",
+			  G_CALLBACK (update_toolbar_remove_flag),
+			  NULL);
 	g_object_set (G_OBJECT (t),
 		      "ToolbarsModel", model,
 		      "MenuMerge", t->priv->ui_merge,
 		      NULL);
 	init_bookmarks_toolbar (t);
-}
 
-static void
-remove_action (EphyToolbarsModel *model,
-	       const char *action_name)
-{
-	int i, n_toolbars, l, n_items;
-
-	n_toolbars = egg_toolbars_model_n_toolbars
-		(EGG_TOOLBARS_MODEL (model));
-
-	for (i = 0; i < n_toolbars; i++)
-	{
-		n_items = egg_toolbars_model_n_items (EGG_TOOLBARS_MODEL (model), i);
-		for (l = 0; l < n_items; l++)
-		{
-			const char *i_name;
-			gboolean sep;
-
-			i_name = egg_toolbars_model_item_nth
-				(EGG_TOOLBARS_MODEL (model), i, l, &sep);
-			if (!sep && strcmp (action_name, i_name) == 0)
-			{
-				egg_toolbars_model_remove_item
-					(EGG_TOOLBARS_MODEL (model), i, l);
-			}
-		}
-	}
-}
-
-static void
-topic_remove_cb (EphyBookmarks *eb,
-		 long id,
-		 Toolbar *t)
-{
-	EggAction *action;
-	char *name;
-	EphyToolbarsModel *model;
-
-	model = ephy_shell_get_toolbars_model (ephy_shell);
-
-	name = g_strdup_printf ("GoTopicId%ld", id);
-	action = egg_action_group_get_action (t->priv->action_group, name);
-	if (action)
-	{
-		remove_action (model, action->name);
-		egg_action_group_remove_action (t->priv->action_group, action);
-	}
-
-	g_free (name);
-}
-
-static void
-bookmark_remove_cb (EphyBookmarks *eb,
-		    long id,
-		    Toolbar *t)
-{
-	EggAction *action;
-	char *name;
-	EphyToolbarsModel *model;
-
-	model = ephy_shell_get_toolbars_model (ephy_shell);
-
-	name = g_strdup_printf ("GoBookmarkId%ld", id);
-	action = egg_action_group_get_action (t->priv->action_group, name);
-	if (action)
-	{
-		remove_action (model, action->name);
-		egg_action_group_remove_action (t->priv->action_group, action);
-	}
-
-	g_free (name);
+	g_signal_connect_object (get_location_entry (t), "user_changed",
+			         G_CALLBACK (location_user_changed_cb),
+			         t, 0);
 }
 
 static void
 toolbar_init (Toolbar *t)
 {
-	EphyBookmarks *bookmarks;
-
         t->priv = g_new0 (ToolbarPrivate, 1);
 
 	t->priv->window = NULL;
 	t->priv->ui_merge = NULL;
 	t->priv->visibility = TRUE;
-
-	bookmarks = ephy_shell_get_bookmarks (ephy_shell);
-	g_signal_connect (bookmarks, "bookmark_remove",
-			  G_CALLBACK (bookmark_remove_cb), t);
-	g_signal_connect (bookmarks, "topic_remove",
-			  G_CALLBACK (topic_remove_cb), t);
+	t->priv->can_set_location = TRUE;
 }
 
 static void
@@ -508,34 +569,35 @@ toolbar_new (EphyWindow *window)
 	return t;
 }
 
-void
-toolbar_edit_location (Toolbar *t)
+static void
+location_finished_cb (GtkWidget *location, GtkWidget *toolbar)
 {
-	EggAction *action;
-	GtkWidget *location;
+	gtk_widget_hide (toolbar);
 
-	action = egg_action_group_get_action
-		(t->priv->action_group, "Location");
-	location = ephy_location_action_get_widget
-		(EPHY_LOCATION_ACTION (action));
-	g_return_if_fail (location != NULL);
-
-	ephy_location_entry_edit
-		(EPHY_LOCATION_ENTRY(location));
+	g_signal_handlers_disconnect_by_func (G_OBJECT (location),
+                                              G_CALLBACK (location_finished_cb),
+                                              toolbar);
 }
 
 void
 toolbar_activate_location (Toolbar *t)
 {
-	EggAction *action;
 	GtkWidget *location;
+	GtkWidget *location_tb;
 
-	action = egg_action_group_get_action
-		(t->priv->action_group, "Location");
-	location = ephy_location_action_get_widget
-		(EPHY_LOCATION_ACTION (action));
+	location = get_location_entry (t);
 	g_return_if_fail (location != NULL);
 
+	location_tb = gtk_widget_get_ancestor (location, EGG_TYPE_TOOLBAR);
+	g_return_if_fail (location_tb != NULL);
+
+	if (!GTK_WIDGET_VISIBLE (location_tb))
+	{
+		g_signal_connect (location, "finished",
+				  G_CALLBACK (location_finished_cb), location_tb);
+		gtk_widget_show (location_tb);
+	}
+
 	ephy_location_entry_activate
 		(EPHY_LOCATION_ENTRY(location));
 }
@@ -566,17 +628,17 @@ void
 toolbar_set_location (Toolbar *t,
 		      const char *alocation)
 {
-	EggAction *action;
 	GtkWidget *location;
 
-	action = egg_action_group_get_action
-		(t->priv->action_group, "Location");
-	location = ephy_location_action_get_widget
-		(EPHY_LOCATION_ACTION (action));
-	g_return_if_fail (location != NULL);
+	if (t->priv->can_set_location)
+	{
+		location = get_location_entry (t);
+		g_return_if_fail (location != NULL);
 
-	ephy_location_entry_set_location
-		(EPHY_LOCATION_ENTRY (location), alocation);
+		ephy_location_entry_set_location
+			(EPHY_LOCATION_ENTRY (location),
+			 alocation ? alocation : "");
+	}
 }
 
 void
@@ -588,7 +650,7 @@ toolbar_update_favicon (Toolbar *t)
 	EggAction *action;
 
 	tab = ephy_window_get_active_tab (t->priv->window);
-	url = ephy_tab_get_favicon_url (tab);
+	url = ephy_tab_get_icon_address (tab);
 	action_group = t->priv->action_group;
 	action = egg_action_group_get_action (action_group, "Favicon");
 	g_object_set (action, "icon", url, NULL);
@@ -597,13 +659,9 @@ toolbar_update_favicon (Toolbar *t)
 char *
 toolbar_get_location (Toolbar *t)
 {
-	EggAction *action;
 	GtkWidget *location;
 
-	action = egg_action_group_get_action
-		(t->priv->action_group, "Location");
-	location = ephy_location_action_get_widget
-		(EPHY_LOCATION_ACTION (action));
+	location = get_location_entry (t);
 	g_return_val_if_fail (location != NULL, NULL);
 
 	return ephy_location_entry_get_location
@@ -613,13 +671,9 @@ toolbar_get_location (Toolbar *t)
 void
 toolbar_clear_location_history (Toolbar *t)
 {
-	EggAction *action;
 	GtkWidget *location;
 
-	action = egg_action_group_get_action
-		(t->priv->action_group, "Location");
-	location = ephy_location_action_get_widget
-		(EPHY_LOCATION_ACTION (action));
+	location = get_location_entry (t),
 	g_return_if_fail (location != NULL);
 
 	ephy_location_entry_clear_history (EPHY_LOCATION_ENTRY (location));
@@ -633,11 +687,11 @@ toolbar_update_navigation_actions (Toolbar *t, gboolean back, gboolean forward,
 
 	action_group = t->priv->action_group;
 	action = egg_action_group_get_action (action_group, "NavigationBack");
-	g_object_set (action, "sensitive", !back, NULL);
+	g_object_set (action, "sensitive", back, NULL);
 	action = egg_action_group_get_action (action_group, "NavigationForward");
-	g_object_set (action, "sensitive", !forward, NULL);
+	g_object_set (action, "sensitive", forward, NULL);
 	action = egg_action_group_get_action (action_group, "NavigationUp");
-	g_object_set (action, "sensitive", !up, NULL);
+	g_object_set (action, "sensitive", up, NULL);
 }
 
 void
@@ -687,3 +741,14 @@ toolbar_set_visibility (Toolbar *t,
 		}
 	}
 }
+
+void
+toolbar_update_zoom (Toolbar *t, float zoom)
+{
+	EggActionGroup *action_group;
+	EggAction *action;
+
+	action_group = t->priv->action_group;
+	action = egg_action_group_get_action (action_group, "Zoom");
+	g_object_set (action, "zoom", zoom, NULL);
+}
